#include <chrono>
#include <iostream>
#include <map>
#include <random>
#include <string>
#include <vector>
#include <algorithm>
#include <execution>
#include "get_cpu_time.h"

using namespace std;
using namespace std::chrono;
constexpr double nb_microseconds_per_milliseconds{1'000.0};

constexpr size_t taille_vecteur{10};
constexpr int valeur_max{100'000'000};

void display_stat(const string& experience,
                  const std::chrono::time_point<system_clock> &start_time, const std::chrono::time_point<system_clock> &stop_time,
                  const unsigned long long &cpu_start_time, const unsigned long long &cpu_stop_time)
{
    const auto duration = duration_cast<milliseconds>(stop_time - start_time);
    const double cpu_time = static_cast<double>(cpu_stop_time - cpu_start_time) / nb_microseconds_per_milliseconds;
    cout << experience << " : " << "elapsed = " << static_cast<double>(duration.count()) << " ms | cpu = " << cpu_time << " ms\n";
}

int main()
{
    std::chrono::time_point<system_clock> start_time;
    std::chrono::time_point<system_clock> stop_time;
    unsigned long long cpu_start_time;
    unsigned long long cpu_stop_time;
    std::random_device rd;
    std::default_random_engine gen(rd());
    std::uniform_int_distribution distrib(0, valeur_max);

    //
    // Initialisation vecteur
    //
    cout << "\nInitialisation vecteur\n";

    // Initialisation vecteur avec push_back
    start_time = system_clock::now();
    cpu_start_time = get_cpu_time();
    // TODO 1 Begin
       // Ecrivez ici votre code pour ce Todo
    // End
    cpu_stop_time = get_cpu_time();
    stop_time = system_clock::now();
    display_stat("Initialisation vecteur avec push_back", start_time, stop_time, cpu_start_time, cpu_stop_time);
    cout << "\tTaille du vecteur = " << v0.size() << "\n";

    // Initialisation vecteur avec reserve taille finale, puis push_back
    start_time = system_clock::now();
    cpu_start_time = get_cpu_time();
    // TODO 2 Begin
       // Ecrivez ici votre code pour ce Todo
    // End
    cpu_stop_time = get_cpu_time();
    stop_time = system_clock::now();
    display_stat("Initialisation vecteur avec reserve taille finale, puis push_back", start_time, stop_time, cpu_start_time, cpu_stop_time);
    cout << "\tTaille du vecteur = " << v1.size() << "\n";

    // Initialisation vecteur avec init sur taille finale, puis affectation
    start_time = system_clock::now();
    cpu_start_time = get_cpu_time();
    // TODO 3 Begin
       // Ecrivez ici votre code pour ce Todo
    // End
    cpu_stop_time = get_cpu_time();
    stop_time = system_clock::now();
    display_stat("Initialisation vecteur avec init sur taille finale, puis affectation", start_time, stop_time, cpu_start_time, cpu_stop_time);
    cout << "\tTaille du vecteur = " << v2.size() << "\n";

    //
    // Fusion de 2 vecteurs
    //
    cout << "\nFusion de 2 vecteurs\n";

    // Ajout de v0 au bout de v1 avec une boucle
    start_time = system_clock::now();
    cpu_start_time = get_cpu_time();
    // TODO 4 Begin
       // Ecrivez ici votre code pour ce Todo
    // End
    cpu_stop_time = get_cpu_time();
    stop_time = system_clock::now();
    display_stat("Ajout de v0 au bout de v1 avec une boucle", start_time, stop_time, cpu_start_time, cpu_stop_time);
    cout << "\tTaille du vecteur = " << v1.size() << "\n";

    // Ajout de v0 au bout de v2 avec un algorithme C++
    start_time = system_clock::now();
    cpu_start_time = get_cpu_time();
    // TODO 5 Begin
       // Ecrivez ici votre code pour ce Todo
    // End
    cpu_stop_time = get_cpu_time();
    stop_time = system_clock::now();
    display_stat("Ajout de v0 au bout de v2 avec un algorithme C++", start_time, stop_time, cpu_start_time, cpu_stop_time);
    cout << "\tTaille du vecteur = " << v2.size() << "\n";

    //
    // Moyenne des éléments de v0
    //
    cout << "\nMoyennes des elements d'un vecteur\n";
    long long somme{0};

    // Calcul moyenne avec une boucle
    start_time = system_clock::now();
    cpu_start_time = get_cpu_time();
    // TODO 6 Begin
       // Ecrivez ici votre code pour ce Todo
    // End
    cpu_stop_time = get_cpu_time();
    stop_time = system_clock::now();
    display_stat("Calcul moyenne avec une boucle", start_time, stop_time, cpu_start_time, cpu_stop_time);
    cout << "\tMoyenne valeurs aleatoires = " << somme / v0.size() << "\n";

    // Calcul moyenne avec accumulate
    start_time = system_clock::now();
    cpu_start_time = get_cpu_time();
    // TODO 7 Begin
       // Ecrivez ici votre code pour ce Todo
    // End
    cpu_stop_time = get_cpu_time();
    stop_time = system_clock::now();
    display_stat("Calcul moyenne avec accumulate", start_time, stop_time, cpu_start_time, cpu_stop_time);
    cout << "\tMoyenne valeurs aleatoires = " << somme / v0.size() << "\n";

    // Calcul moyenne avec reduce
    start_time = system_clock::now();
    cpu_start_time = get_cpu_time();
    // TODO 8 Begin
       // Ecrivez ici votre code pour ce Todo
    // End
    cpu_stop_time = get_cpu_time();
    stop_time = system_clock::now();
    display_stat("Calcul moyenne avec reduce", start_time, stop_time, cpu_start_time, cpu_stop_time);
    cout << "\tMoyenne valeurs aleatoires = " << somme / v0.size() << "\n";

    // Calcul moyenne avec reduce et std::execution::par
    start_time = system_clock::now();
    cpu_start_time = get_cpu_time();
    // TODO 9 Begin
       // Ecrivez ici votre code pour ce Todo
    // End
    cpu_stop_time = get_cpu_time();
    stop_time = system_clock::now();
    display_stat("Calcul moyenne avec reduce et std::execution::par", start_time, stop_time, cpu_start_time, cpu_stop_time);
    cout << "\tMoyenne valeurs aleatoires = " << somme / v0.size() << "\n";

    //
    // Calcul des quartiles des valeurs stockées dans v0
    //
    cout << "\nQuartile des valeurs stockees dans un vecteur\n";

    // Calcul des quartiles en s'appuyant sur les frequences dans une map
    start_time = system_clock::now();
    cpu_start_time = get_cpu_time();
    // TODO 10 Begin
       // Ecrivez ici votre code pour ce Todo
    // End
    cpu_stop_time = get_cpu_time();
    stop_time = system_clock::now();
    display_stat("Calcul des quartiles en s'appuyant sur les frequences dans une map", start_time, stop_time, cpu_start_time, cpu_stop_time);
    cout << "\tQuartiles : Q1 = " << quartile0[0] << " | Q2 = " << quartile0[1] << " | Q3 = " << quartile0[2] << "\n";

    // Calcul des quartiles en s'appuyant sur un tri de v0
    start_time = system_clock::now();
    cpu_start_time = get_cpu_time();
    // TODO 11 Begin
       // Ecrivez ici votre code pour ce Todo
    // End
    cpu_stop_time = get_cpu_time();
    stop_time = system_clock::now();
    display_stat("Calcul des quartiles en s'appuyant sur un tri de v0", start_time, stop_time, cpu_start_time, cpu_stop_time);
    cout << "\tQuartiles : Q1 = " << quartile1[0] << " | Q2 = " << quartile1[1] << " | Q3 = " << quartile1[2] << "\n";

}















